#ifndef CMESHQUALITY_INCLUDED
#define CMESHQUALITY_INCLUDED

#include <boost/utility.hpp>
#include <boost/shared_ptr.hpp>

#include <VertexPool.h>

class CMeshQuality
	: boost::noncopyable
{
public:
	enum TRIANGLE_QUALITY {
		RADII_RATIO,
		EDGE_RATIO,
		ASPECT_RATIO,
		CIRCUMRADIUS_TO_MAX_EDGE_RATIO,
		CIRCUMRADIUS_TO_SEMIPERIMETER_RATIO,
		NORMALIZED_AREA
	};
	CMeshQuality(void);
	virtual ~CMeshQuality(void);
	bool Load(boost::shared_ptr<CVertexPool> mesh);
	bool SetMeasure(TRIANGLE_QUALITY qualityMeasure);
	bool MeasureQuality(void);
	float GetVertexQuality(int vertexName) const;
	float GetMinQuality(void) const;
	float GetMaxQuality(void) const;
	float GetAverageQuality(void) const;
protected:
	void Create(void);
	bool MeasureQualityOfTriangleMesh(void);
	bool MeasureQualityOfQuadMesh(void);
	float GetTriangleQuality(const float *A, const float *B, const float *C) const;
private:
	boost::shared_ptr<CVertexPool> m_mesh;
	TRIANGLE_QUALITY m_qualityMeasure;
	std::vector<float> m_qualityArray;
	float m_minQuality;
	float m_maxQuality;
	float m_averageQuality;
	int m_degenerateTriangleCount;
};

#endif // CMESHQUALITY_INCLUDED

