#ifndef PINHOLECAMERA_INCLUDED
#define PINHOLECAMERA_INCLUDED

class CPinholeCamera
{
public:
	enum TYPE {
		TRACKBALL,
		FIRST_PERSON
	};
	CPinholeCamera(void);
	CPinholeCamera(float fieldOfView, float zNear, float zFar, int windowWidth, int windowHeight);
	CPinholeCamera(const CPinholeCamera &pinholeCamera);
	~CPinholeCamera(void);
	CPinholeCamera &operator=(const CPinholeCamera &pinholeCamera);
	bool Create(float fieldOfView, float zNear, float zfar, int windowWidth, int windowHeight);
	void Pitch(float angle);
	void Yaw(float angle);
	void MoveFront(float step);
	void MoveSide(float step);
	void MoveUp(float step);
	void ApplyTransform(void);
	void SetCameraType(TYPE type);
	void SetViewport(int viewportWidth, int viewportHeight);
	TYPE GetCameraType(void) const;
	void ResetViewMatrix(void);
	void SetViewMatrix(const float *viewMatrix);
	const float *GetViewMatrix(void) const;
	const float *GetProjectionMatrix(void) const;
	const float *GetImagePlaneMatrix(void) const;
	const float *GetCenterOfProjection(void) const;
private:
	void Create(void);
	void Copy(const CPinholeCamera &pinholeCamera);
	void ResetProjectionMatrix(void);
	void CalculateProjectionMatrix(void);
	void ResetImagePlaneMatrix(void);
	void CalculateImagePlaneMatrix(void);
protected:
	TYPE m_type;  
	float m_fieldOfView;
	float m_zNear;
	float m_zFar;
	int m_windowWidth;
	int m_windowHeight;
	float m_pitchRotation;
	float m_yawRotation;
	float m_frontDisplacement;
	float m_sideDisplacement;
	float m_upDisplacement;
	float m_viewMatrix[16];
	float m_projectionMatrix[16];
	float m_imagePlaneMatrix[9];
	float m_centerOfProjection[3];
};

#endif // PINHOLECAMERA_INCLUDED

