#ifdef _WIN32
#include <windows.h>
#endif // _WIN32

#include <stdexcept>
#include <cmath>

#include <float.h>
#include <limits.h>

#include <MyMath.h>

using namespace std;

float CMyMath::m_floatEpsilon = 0.0f;

CMyMath* CMyMath::m_instance = 0;
bool CMyMath::m_isDestroyed = false;

/**
*/
float CMyMath::FloatEpsilon(void)
{
	if (m_floatEpsilon == 0.0f)
		CalculateFloatEpsilon();

	return m_floatEpsilon;
}

/**
*/
float CMyMath::FloatMax(void)
{
	return FLT_MAX;
}

/**
*/
float CMyMath::FloatMin(void)
{
	return -FLT_MAX;
}

/**
*/
float CMyMath::FloatRandom(void)
{
	return (float)rand()/RAND_MAX;
}

/**
*/
bool CMyMath::IsFloatValid(float n)
{
#ifdef _MSC_VER
	switch (_fpclass(n))
	{
	// Negative normalized non-zero.
	case _FPCLASS_NN:
	// Negative denormalized.
	case _FPCLASS_ND:
	// Negative zero (-0).
	case _FPCLASS_NZ:
	// Positive 0 (+0).
	case _FPCLASS_PZ:
	// Positive denormalized.
	case _FPCLASS_PD:
	// Positive normalized non-zero.
	case _FPCLASS_PN:
		return true;
	// Signaling NaN.
	case _FPCLASS_SNAN:
	// Quiet NaN.
	case _FPCLASS_QNAN:
	// Negative infinity (-INF).
	case _FPCLASS_NINF:
	// Positive infinity (+INF).
	case _FPCLASS_PINF:
		return false;
	}
#else // !_MSC_VER
	return true;
#endif // _MSC_VER

	return false;
}

/**
*/
double CMyMath::Pi(void)
{
	return 3.1415926535897932384626433832795;
}

/**
*/
short CMyMath::ShortMax(void)
{
	return SHRT_MAX;
}

/**
*/
CMyMath::CMyMath(void)
{
	CalculateFloatEpsilon();
}

/**
*/
CMyMath::~CMyMath(void)
{
	m_instance = 0;
	m_isDestroyed = true;
}

/**
*/
void CMyMath::Create(void)
{
	static CMyMath theInstance;

	m_instance = &theInstance;
}

/**
*/
void CMyMath::OnDeadReference(void)
{
	throw runtime_error("Dead Reference Detected.");
}

/**
*/
void CMyMath::CalculateFloatEpsilon(void)
{
	float sum;

	m_floatEpsilon = 1.0f;
	
	do 
	{
		m_floatEpsilon /= 2.0f;
		
		sum = 1.0f + m_floatEpsilon;
	} while (sum > 1.0f);

	m_floatEpsilon *= 2.0f;
}

